!function() {
	window.CordovaFileFeature = {
		getRoot : function(option) {
			var resultOption = $.extend({
				'onSuccess' : function(root) {},
				'onError' : function() {}
			}, option);
			
			Inner.data.requestFileSystem({
				'onSuccess' : function(fileSystem) {
					resultOption.onSuccess(fileSystem.root);
				},
				'onError' : function(error) {
					console.error('CordovaFile.getRoot() : '
							+ JSON.stringify(error));
					resultOption.onError();
				}
			});
		},
		getDirectory : function(option) {
			var resultOption = $.extend({
				'directoryEntry' : undefined,
				'directoryName' : undefined,
				'neededToCreate' : false,
				'onSuccess' : function(targetDirectoryEntry) {},
				'onNotFound' : function() {},
				'onError' : function(error) {}
			}, option);
			
			var directoryName = resultOption.directoryName;
			var neededToCreate = resultOption.neededToCreate;
			
			resultOption.directoryEntry.getDirectory(directoryName, {
				'create' : neededToCreate
			}, function(targetDirectoryEntry) {
				resultOption.onSuccess(targetDirectoryEntry);
			}, function(error) {
				if (error && error.code == 1) {
					resultOption.onNotFound();
				} else {
					resultOption.onError(error);
				}
			});
		},
		getFile : function(option) {
			var resultOption = $.extend({
				'directoryEntry' : undefined,
				'fileName' : undefined,
				'neededToCreate' : false,
				'onSuccess' : function(targetFileEntry) {},
				'onNotFound' : function() {},
				'onError' : function(error) {}
			}, option);
			
			resultOption.directoryEntry.getFile(resultOption.fileName, {
				'create' : resultOption.neededToCreate
			}, function(targetFileEntry) {
				resultOption.onSuccess(targetFileEntry);
			}, function(error) {
				if (error && error.code == 1) {
					resultOption.onNotFound();
				} else {
					resultOption.onError(error);
				}
			});
		},
    	download : function(option) {
			var resultOption = $.extend({
				'sourceUrl' : undefined,
				'targetFileEntry' : undefined,
				'onSuccess' : function(downloadedFileEntry) {},
				'onError' : function() {},
				'onComplete' : function() {}
			}, option);
			
			var sourceUrl = resultOption.sourceUrl;
			var targetUrl = resultOption.targetFileEntry.toURL();
			
    		var fileTransfer = new FileTransfer();
			fileTransfer.download(resultOption.sourceUrl, targetUrl,
					function(downloadedFileEntry) {
				resultOption.onSuccess(downloadedFileEntry);
				resultOption.onComplete();
			}, function(error) {
				console.error('CordovaFileFeature.download() error : '
						+ JSON.stringify(error));
				resultOption.onError();
				
				resultOption.onComplete();
			});
    	},
    	remove : function(option) {
			var resultOption = $.extend({
				'fileEntry' : undefined,
				'onSuccess' : function() {},
				'onError' : function() {}
			}, option);
    		
			resultOption.fileEntry.remove(function() {
				resultOption.onSuccess();
			}, function(error) {
				console.error('CordovaFileFeature.remove() error : '
						+ JSON.stringify(error));
				resultOption.onError();
			}, function() {
				resultOption.onSuccess();
			});
    	},
    	upload : function(option) {
			var resultOption = $.extend({
				'fileEntry' : undefined,
				'uploadUrl' : undefined,
				'onSuccess' : function(resultData) {},
				'onError' : function() {}
			}, option);
			
			var fileEntry = resultOption.fileEntry;
			var uploadUrl = resultOption.uploadUrl;
			
			var fileTransfer = new FileTransfer();
			var fileUrl = fileEntry.toURL();
			
			var uploadOption = {
				'fileKey' : 'file',
				'fileName' : fileEntry.name,
				'mimeType' : 'image/png'
			};
			
			fileTransfer.upload(fileUrl, uploadUrl, function(result) {
				var resultData = JSON.parse(result.response);
				resultOption.onSuccess(resultData);
			}, function(error) {
				console.error('CordovaFileFeature.upload() error : '
						+ JSON.stringify(error));
				resultOption.onError();
			}, uploadOption);
    	},
    	getCdvFile : function(option) {
			var resultOption = $.extend({
				'filePath' : undefined,
				'onSuccess' : function(targetFileEntry) {},
				'onError' : function() {}
			}, option);
    		
			window.resolveLocalFileSystemURL(resultOption.filePath, function(targetFileEntry) {
				resultOption.onSuccess(targetFileEntry);
			}, function(error) {
				console.error('CordovaFileFeature.getCdvFile() error : '
						+ JSON.stringify(error));
				resultOption.onError();
			});
    	}
	};
	var Inner = {
		data : {
			requestFileSystem : function(option) {
				var resultOption = $.extend({
					'onSuccess' : function(fileSystem) {},
					'onError' : function(error) {}
				}, option);
				
				window.requestFileSystem(LocalFileSystem.TEMPORARY, 0,
						function(fileSystem) {
					resultOption.onSuccess(fileSystem);
				}, function(error) {
					resultOption.onError(error);
				});
			},
			getTempDirectory : function(option) {
				var resultOption = $.extend({
					'onSuccess' : function(tempDirectory) {},
					'onError' : function() {}
				}, option);
				
				window.requestFileSystem(window.TEMPORARY, 0,
						function(fileSystem) {
					resultOption.onSuccess(fileSystem.root);
				}, function(error) {
					console.error('CordovaFileFeature Inner.data.getTempDirectory() error : '
							+ JSON.stringify(error));
					resultOption.onError(error);
				});
			}
		}
	};
	
	var DateUtil = {
		toString : function(date, option) {
			var resultOption = $.extend({
				'format' : 'yyyy-MM-dd HH:mm:ss'
			}, option);
			
			if (!date) {
				return '';
			}
			
			var year = date.getFullYear();
			var month = date.getMonth() + 1;
			var day = date.getDate();
			var hour = date.getHours();
			var minute = date.getMinutes();
			var second = date.getSeconds();
			
			var dateString = '';
			var neededToPlaceZero = true;
			switch (resultOption.format) {
			case 'yyyy-MM-dd HH:mm:ss':
				dateString = year + '-' + month + '-' + day + ' ' + hour + ':' + minute + ':' + second;
				break;
			case 'yyyy-MM-dd HH:mm':
				dateString = year + '-' + month + '-' + day + ' ' + hour + ':' + minute;
				break;
			case '周w yyyy-MM-dd HH:mm':
				var weekText = DateUtil.getWeekText(date);
				dateString = '' + weekText + ' ' + year + '-' + month + '-' + day + ' ' + hour + ':' + minute;
				break;
			case 'yyyy-MM':
				dateString = year + '-' + month;
				break;
			case 'yyyy-MM-dd':
				dateString = year + '-' + month + '-' + day;
				break;
			case 'yyyy-M-d':
				dateString = year + '-' + month + '-' + day;
				neededToPlaceZero = false;
				break;
			case 'yyyy-MM-dd 周w':
				var weekText = DateUtil.getWeekText(date);
				dateString = year + '-' + month + '-' + day + ' ' + weekText;
				break;
			case 'yyyy-MM-dd_HH-mm-ss':
				dateString = year + '-' + month + '-' + day + '_' + hour + '-' + minute + '-' + second;
				break;
			case 'yyyy年M月d日':
				dateString = year + '年' + month + '月' + day + '日';
				neededToPlaceZero = false;
				break;
			case 'M月d日':
				dateString = month + '月' + day + '日';
				neededToPlaceZero = false;
				break;
			case 'MM-dd':
				dateString = month + '-' + day;
				break;
			case 'MM-dd 周w':
				var weekText = DateUtil.getWeekText(date);
				dateString = month + '-' + day + ' ' + weekText + '';
				break;
			case 'HH:mm:ss':
				dateString = hour + ':' + minute + ':' + second;
				break;
			case 'HH:mm':
				dateString = hour + ':' + minute;
				break;
			}
			
			if (neededToPlaceZero) {
				dateString = DateUtil.fillZero(dateString);
			}
			return dateString;
		},
		fillZero : function(string) {
			var result = string.replace(/\b(\w)\b/g, '0$1');
			return result;
		}
	}
}();
