!function() {
	/**
	 * 1) 前端提供 urls 给 Android，
	 *    过滤出未下载文件的 urls（与本地『已下载文件列表』文件比对），返回给前端
	 * 2) 前端将未下载文件的 urls 给后端，
	 *    后端下载文件，压缩成多个 zip 文件，并上传 OSS 得到 zipUrls，返回给前端
	 * 3) 前端循环将 zipUrls 给 Android，
	 *    下载多个 zip 文件，返回本地路径给前端
	 * 4) 前端将多个 zip 文件的本地路径给 Android，
	 *    解压缩多个 zip 文件，部署资源文件
	 *    （至此步骤，会调用 onLoad 方法）
	 * 5) 记录新下载文件，
	 *    在本地『已下载文件列表』文件中追加新下载文件 url
	 */
	window.LoadFeature = {
		loadFile : function(option) {
			var resultOption = $.extend({
				'fileUrl' : undefined,
				'onLoad' : () => {},
				'onError' : () => {}
			}, option);

			var fileUrl = resultOption.fileUrl;

			LoadFeature.loadFiles({
				'fileUrls' : [ fileUrl ],
				'onAllLoad' : function() {
					resultOption.onLoad();
				},
				'onError' : function() {
					resultOption.onError();
				}
			});
		},
		loadFiles : function(option) {
			var resultOption = $.extend({
				'fileUrls' : [],
				'onStep' : (finishedStep, totalStep) => {},
				'onLoad' : (newDownloadedFileUrls) => {},
				'onError' : () => {}
			}, option);

			var fileUrls = resultOption.fileUrls;

			// 1) 前端提供 urls 给 Android，
			//    过滤出未下载文件的 urls（与本地『已下载文件列表』文件比对），返回给前端
			var step = 1;
			var fromDate = new Date();
			console.log('loadFiles ready, fileUrls.length=' + resultOption.fileUrls.length + '---');
			Inner.android.filterNotDownloaded({
				'fileUrls' : resultOption.fileUrls,
				'onSuccess' : (notDownloadedFileUrls) => {
					console.log('loadFiles : step=' + step + ', durationMs=' + (new Date().getTime() - fromDate.getTime())
							+ ', notDownloadedFileUrls.length=' + notDownloadedFileUrls.length);
					if (notDownloadedFileUrls.length == 0) {
						resultOption.onStep(step, 1);
						resultOption.onLoad(notDownloadedFileUrls);
						return;
					} else {
						resultOption.onStep(step++, 4);
					}

					// 2) 前端将未下载文件的 urls 给后端
					//    后端下载文件，压缩成多个 zip 文件，并上传 OSS 得到 zipUrls，返回给前端
					Inner.data.downloadToZipsWithCache({
						'fileUrls' : notDownloadedFileUrls,
						'onSuccess' : (zipUrls) => {
							resultOption.onStep(step++, 4);
							console.log('loadFiles : step=' + step + ' - durationMs=' + (new Date().getTime() - fromDate.getTime()));

							// 3) 前端循环将 zipUrls 给 Android，
							//    下载多个 zip 文件，返回本地路径给前端
							let downloadZipCount = 0;
							$.each(zipUrls, (index, zipUrl) => {
								Inner.data.download({
									'fileUrl' : zipUrl,
									'onSuccess' : function(fileEntry) {
										downloadZipCount++;
										const isAllZipsDownloaded = (downloadZipCount == zipUrls.length);
										if (isAllZipsDownloaded) {
											resultOption.onStep(step++, 4);
											console.log('loadFiles : step=' + step + ' - durationMs=' + (new Date().getTime() - fromDate.getTime()));
										}

										// 4) 前端将多个 zip 文件的本地路径给 Android，
										//    解压缩多个 zip 文件，部署资源文件
										var zipPath = fileEntry.fullPath;
										Inner.android.decompress({
											'zipPath' : zipPath,
											'onSuccess' : () => {
												if (isAllZipsDownloaded) {
													resultOption.onStep(step++, 4);
													console.log('loadFiles : step=' + step + ' - durationMs=' + (new Date().getTime() - fromDate.getTime()));
													resultOption.onLoad(notDownloadedFileUrls);

													// 5) 记录新下载文件，
													//    在本地『已下载文件列表』文件中追加新下载文件 url
													Inner.android.recordDownloadedFileUrls({
														'downloadedFileUrls' : notDownloadedFileUrls
													});
												}
											},
											'onError' : function() {
												resultOption.onError();
											}
										});
									},
									'onError' : function() {
										resultOption.onError();
									}
								});
							});
						},
						'onError' : function() {
							resultOption.onError();
						}
					});
				},
				'onError' : function() {
					resultOption.onError();
				}
			});
		}
	};

	var Inner = {
		variable : {
			'tempFileNameKey' : 'load',
			'loadFileUrlsFileName' : 'load-file-urls.txt'
		},
		cache : {
			'loadOption' : undefined
		},
		data : {
			download : function(option) {
				var resultOption = $.extend({
					'fileUrl' : undefined,
					'onSuccess' : function(fileEntry) {},
					'onError' : function() {}
				}, option);

				var fileUrl = resultOption.fileUrl;

				FileFeature.getTempDirectory({
					'fileNameKey' : Inner.variable.tempFileNameKey,
					'onSuccess' : function(tempDirectory) {
						var filePath = Inner.file.toFilePath(fileUrl);
						var directoryPath = Inner.file.toDirectoryPath(filePath);
						var fileName = Inner.file.toFileName(filePath);

						FileFeature.getDirectoryByPath({
							'parentDirectoryEntry' : tempDirectory,
							'directoryPath' : directoryPath,
							'onSuccess' : function(directoryEntry) {
								FileFeature.download({
									'fileUrl' : fileUrl,
									'directoryEntry' : directoryEntry,
									'fileName' : fileName,
									'onSuccess' : function(downloadedFileEntry) {
										resultOption.onSuccess(downloadedFileEntry);
									},
									'onError' : function() {
										resultOption.onError();
									}
								});
							},
							'onError' : function() {
								resultOption.onError();
							}
						});
					},
					'onError' : function() {
						resultOption.onError();
					}
				});
			},
			isImageFile : function(fileName) {
				if (StringUtil.contains(fileName, '.jpg')
						|| StringUtil.contains(fileName, '.jpeg')
						|| StringUtil.contains(fileName, '.png')) {
					return true;
				} else {
					return false;
				}
			},
			downloadToZipsWithCache : function(option) {
				var resultOption = $.extend({
					'fileUrls' : undefined,
					'onSuccess' : (zipUrls) => {},
					'onError' : () => {}
				}, option);

				var width = screen.availWidth;
				var height = screen.availHeight;
				var dpr = window.devicePixelRatio;
				AjaxUtil.execute({
					'url' : '/download/download-to-zips-with-cache.do',
					'data' : {
						'fileUrls' : resultOption.fileUrls,
						'deviceWidth' : width,
						'deviceHeight' : height,
						'dpr' : dpr
					},
					'success' : function(data) {
						resultOption.onSuccess(data.zipUrls);
					},
					'error' : function() {
						resultOption.onError();
					}
				});
			}
		},
		android : {
			filterNotDownloaded : function(option) {
				var resultOption = $.extend({
					'fileUrls' : undefined,
					'onSuccess' : (notDownloadedFileUrls) => {},
					'onError' : () => {}
				}, option);
				
				var notDownloadedFileUrls = [];
				FileFeature.getTempDirectory({
					'fileNameKey' : Inner.variable.tempFileNameKey,
					'onSuccess' : function(tempDirectory) {
						FileFeature.getFile({
							'directoryEntry' : tempDirectory,
							'fileName' : Inner.variable.loadFileUrlsFileName,
							'neededToCreate' : true,
							'onSuccess' : function(targetFileEntry) {
		                        FileFeature.readText({
		                            'fileEntry' : targetFileEntry,
		                            'onSuccess' : function(text) {
		                            	var fileUrlsLoaded = text.split('\n');
		                            	
		                            	var notDownloadedFileUrls = [];
		                            	$.each(resultOption.fileUrls, (index, fileUrl) => {
		                            		var loaded = ArrayUtil.contains({
		                            			array: fileUrlsLoaded,
		                            			value: fileUrl
		                            		});
		                            		if (!loaded) {
		                            			notDownloadedFileUrls.push(fileUrl);
		                            		}
		                            	});
		                                resultOption.onSuccess(notDownloadedFileUrls);
		                            },
		                            'onError' : function() {
		                                resultOption.onError();
		                            }
		                        });
							},
							'onError' : function(error) {
								resultOption.onError();
							}
						});
					}
				});
			},
			decompress : function(option) {
				var resultOption = $.extend({
					'zipPath' : undefined,
					'onSuccess' : () => {},
					'onError' : () => {}
				}, option);
				
				CordovaPlugin.execute({
		            'serviceName' : 'FilePlugin',
		            'methodName' : 'decompressZip',
		            'parameters' : [ resultOption.zipPath ],
		            'onSuccess' : function(result) {
		            	resultOption.onSuccess();
		            },
		            'onError' : function(error) {
		            	resultOption.onError();
		            }
				});
			},
			recordDownloadedFileUrls : function(option) {
				var resultOption = $.extend({
					'downloadedFileUrls' : undefined
				}, option);
				
				FileFeature.getTempDirectory({
					'fileNameKey' : Inner.variable.tempFileNameKey,
					'onSuccess' : function(tempDirectory) {
						FileFeature.getFile({
							'directoryEntry' : tempDirectory,
							'fileName' : Inner.variable.loadFileUrlsFileName,
							'neededToCreate' : true,
							'onSuccess' : function(targetFileEntry) {
								var text = resultOption.downloadedFileUrls.join('\n');
								text += '\n';
		                        FileFeature.writeText({
		            				'fileEntry': targetFileEntry,
		            				'text' : text,
		            				'isAppended' : true
		                        });
							}
						});
					}
				});
			}
		},
		file : {
			toFilePath : function(fileUrl) {
				var filePath = fileUrl.replace(/.+aliyuncs[.]com\//, '');
				filePath = filePath.replace(/\?.+/, '');
				return filePath;
			},
			toFileName : function(filePath) {
				var fileName = filePath.replace(/.+\//, '');
				return fileName;
			},
			toDirectoryPath : function(filePath) {
				var directoryPath = '';
				var hasMark = false;
				for (var i = filePath.length - 1; i >= 0; i--) {
					var char = filePath[i];
					if (char == '/') {
						hasMark = true;
					}
					
					if (hasMark) {
						directoryPath = char + directoryPath;
					}
				}
				return directoryPath;
			}
		}
	};
}();
