!function() {
	window.MediaRecordFeature = function(option) {
		var resultOption = $.extend({
			'filePath' : undefined, // 当 filePath 为空时，即为创建新的录音文件；否则，即修改旧的录音文件
			'onPlayStopped' : function() {} // 当播放结束时，触发
		}, option);
		
		var self = this;
		self.option = resultOption;
		self.cache = {
			'media' : undefined,
			'mediaStatus' : Media.MEDIA_NONE
		};
	};
	MediaRecordFeature.prototype = {
		ready : function(option) {
			var resultOption = $.extend({
				'onSuccess' : function() {},
				'onError' : function() {}
			}, option);
			
			var self = this;
			var cache = self.cache;
			var selfOption = self.option;
			var filePath = selfOption.filePath;

			if (filePath) {
				Inner.data.initMedia({
					'filePath' : filePath,
					'self' : self
				});
				
				Inner.data.requestPermission({
					'beforeRequest' : function(hasPermission) {
						if (hasPermission) {
							resultOption.onSuccess();
						}
					}
				});
			} else {
				FileFeature.createTempFile({
					'fileNameKey' : Inner.variable.tempFileNameKey,
					'suffix' : 'mp3',
					'onSuccess' : function(fileEntry) {
						cache.fileEntry = fileEntry;
						
						var newFilePath = fileEntry.toInternalURL();
						Inner.data.initMedia({
							'filePath' : newFilePath,
							'self' : self
						});
						
						Inner.data.requestPermission({
							'beforeRequest' : function(hasPermission) {
								if (hasPermission) {
									resultOption.onSuccess();
								}
							}
						});
					},
					'onError' : function() {
						resultOption.onError();
					}
				});
			}
		},
		startRecord : function() {
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			media.startRecord();
		},
		stopRecord : function() {
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			media.stopRecord();
		},
		startPlay : function() {
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			if (cache.mediaStatus == Media.MEDIA_NONE
					|| cache.mediaStatus == Media.MEDIA_STOPPED) {
				media.play();
			}
		},
		stopPlay : function() {
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			if (cache.mediaStatus == Media.MEDIA_RUNNING) {
				media.stop();
			}
		},
		getDurationSeconds : function() {
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			var duration = media.getDuration();
			return duration;
		},
		getCurrentPositionSeconds : function(option) {
			var resultOption = $.extend({
				'onSuccess' : function(positionSeconds) {}
			}, option);
			
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			media.getCurrentPosition(function(position) {
				if (position >= 0) {
					resultOption.onSuccess(position);
				}
			});
		},
		getFilePath : function() {
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			return media.src;
		},
		getCurrentAmplitude : function(option) {
			var resultOption = $.extend({
				'onSuccess' : function(amplitude) {}
			}, option);
			
			var self = this;
			var cache = self.cache;
			var media = cache.media;
			
			media.getCurrentAmplitude(function(amplitude) {
				resultOption.onSuccess(amplitude);
			});
		}
	};
	$.extend(MediaRecordFeature, {
		upload : function(option) {
			var resultOption = $.extend({
				'filePath' : undefined,
				'onSuccess' : function(fileUrl) {},
				'onError' : function() {}
			}, option);
			
			FileFeature.getCdvFile({
				'filePath' : resultOption.filePath,
				'onSuccess' : function(targetFileEntry) {
					var domain = UrlUtil.getBackServerDomain();
					var uploadUrl = domain + '/audio/upload.do';
					FileFeature.upload({
						'fileEntry' : targetFileEntry,
						'uploadUrl' : uploadUrl,
						'onSuccess' : function(result) {
							resultOption.onSuccess(result.audioUrl);
						},
						'onError' : function() {
							resultOption.onError();
						}
					});
				},
				'onError' : function() {
					resultOption.onError();
				}
			});
		},
		download : function(option) {
			var resultOption = $.extend({
				'fileUrl' : undefined,
				'filePath' : undefined,
				'onSuccess' : function(newFilePath) {
					// 此处的 newFilePath 依次有如下可能
					// 1) 当本地存在 mp3 文件时，不下载，直接返回其  filePath
					// 2) 当本地没有音频文件时，根据 fileUrl 下载（为 mp3 文件），返回其 filePath
				},
				'onError' : function() {}
			}, option);
			
			var fileUrl = resultOption.fileUrl;
			
			FileFeature.getTempDirectory({
				'fileNameKey' : Inner.variable.tempFileNameKey,
				'onSuccess' : function(tempDirectory) {
					var oldTempFileName = resultOption.filePath.replace(/.*\//, '');
					
					FileFeature.getFile({
						'directoryEntry' : tempDirectory,
						'fileName' : oldTempFileName,
						'onSuccess' : function(targetFileEntry) {
							var newFilePath = targetFileEntry.toInternalURL();
							resultOption.onSuccess(newFilePath);
						},
						'onNotFound' : function() {
							var newTempFileNameSuffix = fileUrl.replace(/.*[.]/, '');
							var newTempFileName = oldTempFileName.replace(/[.]wav$/,
									'.' + newTempFileNameSuffix);
							FileFeature.getFile({
								'directoryEntry' : tempDirectory,
								'fileName' : newTempFileName,
								'onSuccess' : function(targetFileEntry) {
									var filePath = targetFileEntry.toInternalURL();
									resultOption.onSuccess(filePath);
								},
								'onNotFound' : function() {
									FileFeature.download({
										'fileUrl' : fileUrl,
										'directoryEntry' : tempDirectory,
										'fileName' : newTempFileName,
										'onSuccess' : function(downloadedFileEntry) {
											var downloadedFilePath =
													downloadedFileEntry.toInternalURL();
											resultOption.onSuccess(downloadedFilePath);
										},
										'onError' : function() {
											resultOption.onError();
										}
									});
								},
								'onError' : function(error) {
									resultOption.onError();
								}
							});
						},
						'onError' : function(error) {
							resultOption.onError();
						}
					});
				},
				'onError' : function() {
					resultOption.onError();
				}
			});
		}
	});
	
	var Inner = {
		variable : {
			'tempMediaFileName' : 'media.tmp',
			'speechPluginName' : 'SpeechPlugin',
			'commaDelayMs' : 1000,
			'periodDelayMs' : 5000,
			'tempFileNameKey' : 'voice'
		},
		cache : {
			'hasPermission' : undefined
		},
		data : {
			initMedia : function(option) {
				var resultOption = $.extend({
					'filePath' : undefined,
					'self' : undefined
				}, option);
				
				var self = resultOption.self;
				var cache = self.cache;
				var selfOption = self.option;
				
				var filePath = resultOption.filePath;
				
				cache.media = new Media(filePath, function() {}, function(e) {
    				console.error('feature/media-record initMedia() error : '
    						+ JSON.stringify(e));
				}, function(mediaStatus) {
					cache.mediaStatus = mediaStatus;
					if (mediaStatus == Media.MEDIA_STOPPED) {
						selfOption.onPlayStopped();
					}
				});
			},
			requestPermission : function(option) {
				var resultOption = $.extend({
					'beforeRequest' : function(hasPermission) {}
				}, option);
				
				if (Inner.cache.hasPermission) {
					resultOption.beforeRequest(true);
				} else {
					CordovaPlugin.execute({
			            'serviceName' : 'PermissionPlugin',
			            'methodName' : 'requestPermissions',
			            'parameters' : [ [ 'android.permission.RECORD_AUDIO' ] ],
			            'onSuccess' : function(result) {
			            	var hasPermission = !result.neededToRequest;
			            	resultOption.beforeRequest(hasPermission);
			            	
			            	Inner.cache.hasPermission = hasPermission;
			            }
					});
				}
			}
		},
		dom : {
			start : function(option) {
				var resultOption = $.extend({
					'onSuccess' : function() {},
					'onError' : function() {}
				}, option);
				
				navigator.device.capture.captureAudio(
					function() {
						resultOption.onSuccess();
					}, function(e) {
						console.error(JSON.stringify(e));
						resultOption.onError(e);
					}
				);
			}
		}
	};
}();
