!function() {
	window.StringUtil = {
		trim : function(string) {
			if (string) {
				var result = string.replace(/(^\s*)|(\s*$)/g, '');
				result = result.replace(/\u200B/g, '');
				return result;
			} else {
				return string;
			}
		},
		startsWith : function(sourceString, queryString) {
			var pattern = new RegExp('^' + queryString, "g");
			return pattern.test(sourceString);
		},
		endsWith : function(sourceString, queryString) {
			var pattern = new RegExp(queryString + "$", "g");
			return pattern.test(sourceString);
		},
		contains : function(sourceString, queryString) {
			return (sourceString.indexOf(queryString) != -1);
		}
	};
	window.LoopUtil = {
		loop : function(option) {
			var resultOption = $.extend({
				'onLoop' : function(toContinue) {},
				'durationMs' : 1000,
				'neededExecuteRightNow' : true
			}, option);
			
			var loopAction = function(newDurationMs) {
				var durationMs = (newDurationMs > 0
						? newDurationMs
						: resultOption.durationMs);
				setTimeout(function() {
					resultOption.onLoop(function(durationMs) {
						loopAction(durationMs);
					});
				}, durationMs);
			};
			if (resultOption.neededExecuteRightNow) {
				resultOption.onLoop(function(newDurationMs) {
					loopAction(newDurationMs);
				});
			} else {
				setTimeout(function() {
					resultOption.onLoop(function(newDurationMs) {
						loopAction(newDurationMs);
					});
				}, resultOption.durationMs);
			}
		}
    };
	window.DateUtil = {
		toString : function(date, option) {
			var resultOption = $.extend({
				'format' : 'yyyy-MM-dd HH:mm:ss'
			}, option);
			
			if (!date) {
				return '';
			}
			
			var year = date.getFullYear();
			var month = date.getMonth() + 1;
			var day = date.getDate();
			var hour = date.getHours();
			var minute = date.getMinutes();
			var second = date.getSeconds();
			
			var dateString = '';
			var neededToPlaceZero = true;
			switch (resultOption.format) {
			case 'yyyy-MM-dd HH:mm:ss':
				dateString = year + '-' + month + '-' + day + ' ' + hour + ':' + minute + ':' + second;
				break;
			case 'yyyy-MM-dd HH:mm':
				dateString = year + '-' + month + '-' + day + ' ' + hour + ':' + minute;
				break;
			case '周w yyyy-MM-dd HH:mm':
				var weekText = DateUtil.getWeekText(date);
				dateString = '' + weekText + ' ' + year + '-' + month + '-' + day + ' ' + hour + ':' + minute;
				break;
			case 'yyyy-MM':
				dateString = year + '-' + month;
				break;
			case 'yyyy-MM-dd':
				dateString = year + '-' + month + '-' + day;
				break;
			case 'yyyy-M-d':
				dateString = year + '-' + month + '-' + day;
				neededToPlaceZero = false;
				break;
			case 'yyyy-MM-dd 周w':
				var weekText = DateUtil.getWeekText(date);
				dateString = year + '-' + month + '-' + day + ' ' + weekText;
				break;
			case 'yyyy-MM-dd_HH-mm-ss':
				dateString = year + '-' + month + '-' + day + '_' + hour + '-' + minute + '-' + second;
				break;
			case 'yyyy年M月d日':
				dateString = year + '年' + month + '月' + day + '日';
				neededToPlaceZero = false;
				break;
			case 'M月d日':
				dateString = month + '月' + day + '日';
				neededToPlaceZero = false;
				break;
			case 'MM-dd':
				dateString = month + '-' + day;
				break;
			case 'MM-dd 周w':
				var weekText = DateUtil.getWeekText(date);
				dateString = month + '-' + day + ' ' + weekText + '';
				break;
			case 'HH:mm:ss':
				dateString = hour + ':' + minute + ':' + second;
				break;
			case 'HH:mm':
				dateString = hour + ':' + minute;
				break;
			}
			
			if (neededToPlaceZero) {
				dateString = DateUtil.fillZero(dateString);
			}
			return dateString;
		},
		fillZero : function(string) {
			var result = string.replace(/\b(\w)\b/g, '0$1');
			return result;
		}
	};
	window.UrlUtil = {
		getParametersString : function() {
			var parametersString = window.location.search.substr(1);
			parametersString = StringUtil.trim(parametersString);
			return parametersString;
		},
		getParameter : function(parameterKey) {
			var parameters = UrlUtil.getParameters(parameterKey);
			if (parameters) {
				return parameters[0];
			}
		},
		appendParameter : function(sourceUrl, key, value) {
			var parameterMap = {};
			parameterMap[key] = value;
			
			var resultUrl = UrlUtil.appendParameterMap(sourceUrl, {
				'parameterMap' : parameterMap
			});
			return resultUrl;
		},
		replaceParameter : function(parameterKey, parameterValue) {
			var currentUrl = UrlUtil.getCurrentUrl();
			var currentUrlHeader = currentUrl.replace(/\?.*/g, '');
			
			var parameterMap = UrlUtil.getParameterMap();
			parameterMap[parameterKey] = parameterValue;
			
			var newUrl = UrlUtil.appendParameterMap(currentUrlHeader, {
				'parameterMap' : parameterMap
			});
			return newUrl;
		},
		getParameters : function(parameterKey) {
			return UrlUtil.getParameterMap()[parameterKey];
		},
		getParameterMap : function() {
			var parametersString = UrlUtil.getParametersString();
			var parameterMap = {};
			if (parametersString) {
				$.each(parametersString.split('&'), function(){
					var parameterString = this;
					var parameterKeyValue = parameterString.split('=');
					var parameterKey = parameterKeyValue[0];
					var parameterValue = parameterKeyValue[1];
					if (!parameterMap[parameterKey]) {
						parameterMap[parameterKey] = [];
					}
					parameterMap[parameterKey].push(parameterValue);
				});
			}
			return parameterMap;
		},
		appendParameterMap : function(sourceUrl, option) {
			var resultOption = $.extend({
				'parameterMap' : undefined
			}, option);
			
			var resultUrl = sourceUrl;
			var parameterMap = resultOption.parameterMap;
			
			if (sourceUrl.indexOf('?') == -1)
				resultUrl += '?';
			else if (sourceUrl.indexOf('=') != -1 
					&& !StringUtil.endsWith(sourceUrl, '&'))
				resultUrl += '&';
			
			var parameterKeyValues = [];
			$.each(parameterMap, function(key, value) {
				var parameterKeyValue = key + '=' + this;
				parameterKeyValues.push(parameterKeyValue);
			});
			resultUrl += parameterKeyValues.join('&');
			
			if (parameterKeyValues.length > 0) {
				return resultUrl;
			} else {
				return sourceUrl;
			}
		},
		getBackServerDomain : function() {
			var model = UrlUtil.getParameter('model');
			var domain;
			if (model == 'test') {
				domain = 'http://android-back-test.huibenxia.com';
//				domain = 'http://192.168.0.3:81';
			} else {
				domain = 'http://android-back.huibenxia.com';
			}
			return domain;
		},
		prependBackServerDomainIfNeeded : function(url) {
			var resultUrl;
			var hasPrefix = (StringUtil.startsWith(url, 'http://')
					|| StringUtil.startsWith(url, 'https://'));
			if (hasPrefix) {
				resultUrl = url;
			} else {
				var backServerDomain = UrlUtil.getBackServerDomain();
				var resultUrl = backServerDomain;
				if (!StringUtil.startsWith(url, '/')) {
					resultUrl += '/';
				}
				resultUrl += url;
			}
			return resultUrl;
		},
		appendImageResizeSuffix : function(url) {
			if (StringUtil.contains(url, 'x-oss-process')) {
				return url;
			}
			
			var width = $(window).width();
			var height = $(window).height();
			var dpr = window.devicePixelRatio;
			
			var resizePercent = Math.ceil(width * dpr / 3658 * 100);
			url = UrlUtil.appendParameter(url, 'x-oss-process',
					'image/resize,p_' + resizePercent);
			return url;
		}
	};
	window.AjaxUtil = {
		execute	: function(option) {
			var resultOption = $.extend({
				'url' : undefined,
				'data' : {},
				'success' : function(data) {},
				'error' : function() {},
				'complete' : function() {}
			}, {
				'contentType' : 'application/json;',
				'type' : 'POST'
			}, option);
			
			var url = resultOption.url;
			var data = resultOption.data;
			var type = resultOption.type;
			
			if (type == 'POST') {
				if (data && !TypeUtil.isString(data)) {
					resultOption.data = JSON.stringify(data);
				}
			}
			
			resultOption.url = UrlUtil.prependBackServerDomainIfNeeded(url);
			
			$.ajax(resultOption);
		}
	};
	window.TypeUtil = {
		isString : function(data) {
			var isString = (typeof data == 'string');
			isString = isString && (data.constructor == String);
			return isString;
		}
	};
}();
